package sygr3emif;

import java.time.Instant;
import java.util.ArrayList;
import java.util.Collections;

import sygr.pots.extensions.BusinessParameter;
import sygr.pots.extensions.ExtConstants;
import sygr.pots.extensions.PluginData;
import sygr.pots.extensions.PluginInterface;
import sygr.pots.extensions.PluginUtilInterface;
import sygr3em.model.InboundMessage;
import sygr3em.model.MultiTelemetry;
import sygr3em.model.SingleMerge;
import sygr3em.model.SingleProperty;
import sygr3em.model.SingleTelemetry;
import sygr3em.service.S3eUtil;

public class MsgConvMergeExisting implements PluginInterface {
	
	private String LOGPREFIX = "S3eDemo: ";

	@Override
	public void execute(PluginData data, PluginUtilInterface util) {
		
		if(data == null || util == null) return;
		if(data.messageIn == null || data.messageIn.getContent() == null 
				|| data.messageIn.getContent().equals("")) {
			util.log(LOGPREFIX + "no content arrived.");
			return;
		}
		
		// Expectation is to receive a sort of semicolon separate .csv file
		
		/*
		 * Content like:
		 * 
		 * ID;HU;hu-test-xxx;
		 * SRC;HU;hu-test-xxx;X;
		 * SRC;HU;hu-test-xxx;;
		 * ...
		 */
		
		ArrayList<String> lines = new ArrayList<>();
		try {
			String[] pts = data.messageIn.getContent().split(System.lineSeparator());
			Collections.addAll(lines, pts);
		} catch (Exception e) {
			util.log(LOGPREFIX + "content could not split into lines.");
			return;
		}
		
		if(lines == null || lines.size() == 0) {
			util.log(LOGPREFIX + "no lines received.");
			return;
		}
		
		// Now create an input object for the standard sygr3em and fill
		InboundMessage imsg = new InboundMessage();
		
		for(String line: lines) {
			
			// Our separator will be the ";"
			try {
				String[] pts = line.split(";");
				if(pts == null || pts.length == 0) continue;
				
				// OK, let's check what do we have
				
				// ID line
				if(pts[0].equals("ID")) {
					
					if(pts.length < 3) {
						util.log(LOGPREFIX + "ID line is too short: " + line);
						return;
					}
					util.log(LOGPREFIX + "ID key = |" + pts[1] + "|");
					imsg.getId().setKey(pts[1]);
					util.log(LOGPREFIX + "ID value = |" + pts[2] + "|");
					imsg.getId().setValue(pts[2]);
					
				}
				
				// Source line
				if(pts[0].equals("SRC")) {
					
					if(pts.length < 4) {
						util.log(LOGPREFIX + "Source line is too short: " + line);
						return;
					}
					util.log(LOGPREFIX + "Source = |" + pts[1] + "|" + pts[2] + "|" + pts[3] + "|");
					SingleMerge sm = new SingleMerge();
					sm.setKey(pts[1]);
					sm.setValue(pts[2]);
					if(pts[3].equals("X")) sm.setStatus(ExtConstants.potstatusDELETED);
					imsg.getMerges().getList().add(sm);
					
				}
				
			} catch (Exception e) {
				util.log(LOGPREFIX + "line not understandable: " + line);
				return;
			}
			
		}
			
		// Theoretically we are done, just call the standard inbound interface
			
		// Get the standard inbound plugin name
		String inpluginnname = "";
		String commandMergeExisting = "";
		ArrayList<BusinessParameter> bps 
			= util.getBusinessParameters("s3eDemo", "", "", "");
		if(bps == null || bps.size() == 0) {
			util.log(LOGPREFIX + "Business parameter s3eDemo/inboundplugin not maintained.");
			return;
		}
		for(BusinessParameter bp: bps) {
			if(bp.getKey0().equals("inboundplugin")) inpluginnname = bp.getValue();
			if(bp.getKey0().equals("commandMergeExisting")) commandMergeExisting = bp.getValue();
		}
			
		if(inpluginnname == null || inpluginnname.equals("")) {
			util.log(LOGPREFIX + "s3eDemo/inboundplugin is null.");
			return;
		}
		if(commandMergeExisting == null || commandMergeExisting.equals("")) {
			util.log(LOGPREFIX + "s3eDemo/commandSetTask is null.");
			return;
		}
			
		// Set the CreateNew command
		imsg.setCommand(commandMergeExisting);
			
		// Get the standard plugin itself
		PluginInterface pi = util.getPlugin(inpluginnname);
		if(pi == null) {
			util.log(LOGPREFIX + "Standard inbound plugin " + inpluginnname 
				+ " does not exist.");
			return;
		}
			
		// We use the transfer, to speed up
		ArrayList<Object> alo = new ArrayList<>();
		alo.add(imsg);
		data.transfer = alo;
		
		// and call the standard plugin
		pi.execute(data, util);
		
	}

	@Override
	public void reloadConfig(ArrayList<String> changes, PluginUtilInterface util) {}

}
