package sygr3emif;

import java.time.Instant;
import java.util.ArrayList;
import java.util.Collections;

import sygr.pots.extensions.BusinessParameter;
import sygr.pots.extensions.PluginData;
import sygr.pots.extensions.PluginInterface;
import sygr.pots.extensions.PluginUtilInterface;
import sygr3em.model.InboundMessage;
import sygr3em.model.MultiTelemetry;
import sygr3em.model.SingleProperty;
import sygr3em.model.SingleTelemetry;
import sygr3em.service.S3eUtil;

public class MsgConvTelemetry implements PluginInterface {
	
	private String LOGPREFIX = "S3eDemo: ";

	@Override
	public void execute(PluginData data, PluginUtilInterface util) {
		
		if(data == null || util == null) return;
		if(data.messageIn == null || data.messageIn.getContent() == null 
				|| data.messageIn.getContent().equals("")) {
			util.log(LOGPREFIX + "no content arrived.");
			return;
		}
		
		// Expectation is to receive a sort of semicolon separate .csv file
		
		/*
		 * Content like:
		 * 
		 * ID;HU;hu-test-xxx;
		 * SOURCE;message source;
		 * Time;2023-09-05 09:12:54;
		 * Measurement;temperature;23.5;C;
		 * Measurement;humidity;76.43;%;
		 * Time;2023-09-05 10:12:55;
		 * Measurement;temperature;23.5;C;
		 * Measurement;humidity;76.43;%;
		 * ...
		 */
		
		ArrayList<String> lines = new ArrayList<>();
		try {
			String[] pts = data.messageIn.getContent().split(System.lineSeparator());
			Collections.addAll(lines, pts);
		} catch (Exception e) {
			util.log(LOGPREFIX + "content could not split into lines.");
			return;
		}
		
		if(lines == null || lines.size() == 0) {
			util.log(LOGPREFIX + "no lines received.");
			return;
		}
		
		// Now create an input object for the standard sygr3em and fill
		InboundMessage imsg = new InboundMessage();
		// Also create the telemetry place
		ArrayList<MultiTelemetry> measurements  = new ArrayList<>();
		MultiTelemetry multi = null;
		
		for(String line: lines) {
			
			// Our separator will be the ";"
			try {
				String[] pts = line.split(";");
				if(pts == null || pts.length == 0) continue;
				
				// OK, let's check what do we have
				
				// ID line
				if(pts[0].equals("ID")) {
					
					if(pts.length < 3) {
						util.log(LOGPREFIX + "ID line is too short: " + line);
						return;
					}
					util.log(LOGPREFIX + "ID key = |" + pts[1] + "|");
					imsg.getId().setKey(pts[1]);
					util.log(LOGPREFIX + "ID value = |" + pts[2] + "|");
					imsg.getId().setValue(pts[2]);
					
				}
				
				if(pts[0].equals("SOURCE")) {
					
					if(pts.length < 2) {
						util.log(LOGPREFIX + "SOURCE line is too short: " + line);
						return;
					}
					util.log(LOGPREFIX + "SOURCE = |" + pts[1] + "|");
					SingleProperty sp = new SingleProperty();
					sp.setKey(pts[0]);
					sp.setValue(pts[1]);
					imsg.addSecondaryAttribute(sp);
					
					
				}
				
				// SOURCE line
				
				
				// Timestamp line
				if(pts[0].equals("Time")) {
					
					if(pts.length < 2) {
						util.log(LOGPREFIX + "Time line is too short: " + line);
						return;
					}
					String logst = "Time = |" + pts[1] + "|";
					util.log(LOGPREFIX + logst);
					
					// We need to start a new multitelemetry, if we started to fill one
					if(!(multi == null)) measurements.add(multi);
					multi = new MultiTelemetry();
					// Need to convert the time String (pts[1]) to Instant
					multi.setTimestamp(S3eUtil.convertTimestampStringToInstant(pts[1], util));
					
				}
				
				// Measurement line
				if(pts[0].equals("Measurement")) {
					
					if(pts.length < 3) {
						util.log(LOGPREFIX + "Measurement line is too short: " + line);
						return;
					}
					String logst = "Measurement = |" + pts[1] + "| | " + pts[2];
					if(pts.length > 3) logst = logst + " | | " + pts[3];
					util.log(LOGPREFIX + logst);
					
					SingleTelemetry st = new SingleTelemetry();
					st.setC12c(pts[1]);
					st.setValue(S3eUtil.convertStringToNumeric(pts[2], util));
					if(pts.length > 3) st.setUom(pts[3]);
					multi.getList().add(st);
				}
				
				
			} catch (Exception e) {
				util.log(LOGPREFIX + "line not understandable: " + line);
				return;
			}
			
		}
		
		// Last one
		if(!(multi == null)) measurements.add(multi);
			
		// Theoretically we are done, just call the standard inbound interface
			
		// Get the standard inbound plugin name
		String inpluginnname = "";
		String commandTelemetry = "";
		ArrayList<BusinessParameter> bps 
			= util.getBusinessParameters("s3eDemo", "", "", "");
		if(bps == null || bps.size() == 0) {
			util.log(LOGPREFIX + "Business parameter s3eDemo/inboundplugin not maintained.");
			return;
		}
		for(BusinessParameter bp: bps) {
			if(bp.getKey0().equals("inboundplugin")) inpluginnname = bp.getValue();
			if(bp.getKey0().equals("commandTelemetry")) commandTelemetry = bp.getValue();
		}
			
		if(inpluginnname == null || inpluginnname.equals("")) {
			util.log(LOGPREFIX + "s3eDemo/inboundplugin is null.");
			return;
		}
		if(commandTelemetry == null || commandTelemetry.equals("")) {
			util.log(LOGPREFIX + "s3eDemo/commandTelemetry is null.");
			return;
		}
			
		// Set the CreateNew command
		imsg.setCommand(commandTelemetry);
		// Set the measurements
		imsg.setMeasurements(measurements);
			
		// Get the standard plugin itself
		PluginInterface pi = util.getPlugin(inpluginnname);
		if(pi == null) {
			util.log(LOGPREFIX + "Standard inbound plugin " + inpluginnname 
				+ " does not exist.");
			return;
		}
			
		/*
		// If have the plugin, convert the imsg back to a String
		String content = "";
		if(data.jsonMapper == null) {
			util.log(LOGPREFIX + "data.jsonMapper is null.");
			return;
		}
		try {
			content = data.jsonMapper.writeValueAsString(imsg);
		} catch (Exception e) {
			util.log(LOGPREFIX + "Exception when converting the inbound structure to String");
			return;
		}
			
		// Replace the original content
		data.messageIn.setContent(content);
		*/
		// We use the transfer, to speed up
		ArrayList<Object> alo = new ArrayList<>();
		alo.add(imsg);
		data.transfer = alo;
		
		// and call the standard plugin
		pi.execute(data, util);
		
	}

	@Override
	public void reloadConfig(ArrayList<String> changes, PluginUtilInterface util) {}

}
